function[mloga,vloga]=ricker_loga(s,r,c_global,c_local,inc_up,inc_down);
%%%Function to numerically obtain mean and variance of posterior distribution 
%of 'log(alpha)' in the Ricker model,
%R=alpha*S*exp(beta*S)*exp(e),where e~N(0,v):
%This function uses 'ricker', 'ricker2',and 'pdf' to generate the results.

%%%Input:
%'s' is a data for stock
%'r' is a data for recruitment
%'c_global' is used for determining range of 'log(alpha)' and 'beta' at 
%which a probability density is calculated. This range is determine by 
%comparing a probability density to the maximum probability.
%'c_local' is used for determining a grid of 'beta'. The grid is used for 
%marginalizing 'beta' out to obtain a probability density of 'log(alpha)'.
%'inc_up' is used for determining a grid of 'log(alpha)' in a positive 
%direction. 
%'inc_down' is used for determining a grid of 'log(alpha)' in a positive
%direction.

%%%Output:
%'mloga' is a posterior mean of 'log(alpha)'.
%'vloga' is a posterior variance of 'log(alpha)'.

%%%Note:
%Since the noise term,v,in the Ricker model is marginalized out analytically,
%this code first calculates integration of 'beta' in the Ricker model to 
%obtain a probability density function of 'log(alpha)'. Then, this probability 
%density function is used for obtaining the posterior mean and the variance 
%of 'log(alpha)'.
%To run this code, rename data file as 'test_data',copy and paste a script
%taht follows this sentence,and hit 'save and run'.
%clear all;close all;load('test_data');c_global=0.01;c_local=0.75;inc_up=0.5;inc_down=-0.5;[mloga,vloga]=ricker_loga(s,r,c_global,c_local,inc_up,inc_down);

%%%Function begins from here:
%Obtaining maximum a posteriori (MAP) estimates of 'log(alpha)' and 'beta' in
%the Ricker model.
x_ini=fminsearch(@(th)ricker(th,s,r),[1,0]);

%Obtaining the maximum probability density (unnormalized) of a joint
%posterior distributions of 'log(alpha)' and 'beta'. This value is used for
%normalizing the probability so that the integration over 'log(alpha)' and
%'beta' yields 1.
mle=ricker2([x_ini],s,r);

%This section numerically constructs the probability density function of 
%the posterior distribution of 'beta' give a MAP estimate of 'log(alpha)'.
%Then, we integrate over 'beta' to generate the marginalized probability
%density of 'log(alpha)'. We limit the integration over 'beta' at which 
%the probabiity density given a MAP estimate of 'log(alpha)' and a particular 
%value of 'beta' is 0.01 of the maximum probability density evaluate at 
%MAP estimates of 'beta' and 'log(alpha)'.
%'xcond' is a MAP estimate of 'log(alpha)'.
%'xvar' is a MAP estimate of 'beta'.
xcond=x_ini(1);
xvar=x_ini(2);
[xgrid_up,xpdf_up]=pdf(c_global,c_local,inc_up,xvar,xcond,mle,s,r);
[xgrid_down,xpdf_down]=pdf(c_global,c_local,inc_down,xvar,xcond,mle,s,r);
mle_global=sum(xpdf_up)+sum(xpdf_down);

%Constructing a probability density of 'log(alpha)' for entire range of 
%'log(alpha)'. The entire range is determined by a specific value of 
%'log(alpha)' that generate the probability density that is larger than 
%0.01 of the maximum probability density of 'log(alpha)'.To construct a
%grid of 'log(alpha)' in a positive direction,'i=1' is used. To construct a
%grid of 'log(alpha)' in a negative direction, 'i=2' is used.   
for i=1:2;
    %To understand what this section does, see also a function 'pdf'.
    count=0;
    direction=i;
    step=1;    
    ratio_global=1;
    ratio_local=0;
    x_old=xcond;
    like_old=mle_global;
    while ratio_global>c_global;
        %The while loop that follows this comments determines if a new value 
        %of 'log(alpha)' is a reasonably close to a specific value of 'log(alpha)'
        %at which the probability density was constructed. We accept the new 
        %value of 'log(alpha)' if the new probability density is not smaller 
        %than 0.75 (indicated with 'c_local') of the previous probability density 
        %(indicated with 'ratio_local'). 
        while ratio_local<c_local;
        %Generating a next 'log(alpha)' using a some increment 'inc'.
            switch count
                case 0
                    switch direction
                        case 1
                            %The first proposed next 'log(alpha)'.
                            inc=inc_up;                          
                        case 2
                            %The first proposed next 'log(alpha)'.
                            inc=inc_down;
                    end
                    x_new=x_old+inc;
                otherwise
                    %Shrinking the increment till the 0.75 criterion is
                    %met.
                    x_new=1/2*(x_new+x_old);
            end
            %Finding the maximum probability density given new
            %'log(alpha)'. See also a function 'pdf' for details.
            x_free=fminsearch(@(xvar)ricker([x_new,xvar],s,r),x_ini(2));
            mle_local=ricker2([x_new,x_free],s,r);                            
            [xgrid_up,xpdf_up]=pdf(c_global,c_local,inc_up,x_free,x_new,mle_local,s,r);
            [xgrid_down,xpdf_down]=pdf(c_global,c_local,inc_down,x_free,x_new,mle_local,s,r);
            like_new=sum([xpdf_up;xpdf_down]);
            count=count+1;
            ratio_local=like_new/like_old;
        end 
        %Using the grid for 'beta', we calcualte the probability density using
        %the Simpson's rule ('integral'). See also a function 'pdf' for details.
        fold=like_old;
        fnew=like_new;
        x_free=fminsearch(@(xvar)ricker([1/2*(x_old+x_new),xvar],s,r),x_ini(2));
        mle_local=ricker2([x_new,x_free],s,r); 
        [xgrid_up,xpdf_up]=pdf(c_global,c_local,inc_up,x_free,x_new,mle_local,s,r);
        [xgrid_down,xpdf_down]=pdf(c_global,c_local,inc_down,x_free,x_new,mle_local,s,r);
        fmid=sum([xpdf_up;xpdf_down]);
        integral(step,i)=1/6*abs(x_new-x_old)*(fold+4*fmid+fnew);

        %Updating values.
        x(step,i)=1/2*(x_old+x_new); 
        like_old=like_new;     
        x_old=x_new;
        ratio_global=like_old/mle_global;
        step=step+1;
        count=0; 
        ratio_local=0;
    end
end
%Results.
%'cdf' is a normalizing constant.
cdf=sum(integral(:,1))+sum(integral(:,2));
%'momnet1' is the first moment.
moment1=1/cdf*(sum(integral(:,1).*x(:,1))+sum(integral(:,2).*x(:,2)));
%'moment2' is the second moment.
moment2=1/cdf*(sum(integral(:,1).*(x(:,1).^2))+sum(integral(:,2).*(x(:,2).^2)));
%'mloga' is mean of 'log(alpha)' and 'vloga' is variance of 'log(alpha)'.
mloga=moment1;
vloga=moment2-moment1^2;